angular.module('starter.controllers', ['ngStorage'])

.controller('AppController', function($scope, $state, $sce, $timeout, $element, $localStorage, $ionicViewSwitcher, $ionicPlatform, $ionicLoading, $ionicSideMenuDelegate, $cordovaGeolocation, Server, Globals) {
    console.log(" ")
    console.log(" ")
    console.log(" ")
    console.log(" ")
    console.log(" ")
    console.log("App Controller")

    $scope.globals = Globals;

    // current location of the user
    $scope.coordinates = { 
      latitude: null, 
      longitude: null,

      loading: true,
      error: ""
    };

    // current user data
    $scope.user = $localStorage.user;

    $scope.stateData = {
      // name of the last state the user was on before they were sent to the login view
      loginLastState: ""
    }

    $scope.data = {
      stats: null,
      tags: null,
      categories: null,
      types: null
    }

    // current set of entries
    $scope.entries = {
        data: [],
        total: 0,
        lastState: "tab.list"
    }

    $scope.entryData = {
        id: 0,
        error: null,
        loading: false
    }

    // search/filter data
    $scope.searchData = {
        mode: "current",
        current: true,
        lastStateName: "",

        keywords: "",
        location: "",
        typeId: null,
        categoryId: null,
        tagId: null,
        distance: 50,
        max: 100,
        sortBy: "distance",

        longitude: null,
        latitude: null,

        searching: true,
        error: ""
      };

    // get data/stats for the homepage
    Server.data(function(data) {
      console.log("data callback")
      console.log(data)
      $scope.data.types = data.types;
      $scope.data.categories = data.categories;
      $scope.data.tags = data.tags;
      $scope.data.stats = data.stats;
    })

    $scope.openMainWebsite = function() {
      window.open('http://theclio.com', '_system', 'location=yes');
    }
    $scope.openExternalUrl = function(url) {
      window.open(url, '_system', 'location=yes');
    }
    $scope.trustSrc = function(src) {
      return $sce.trustAsResourceUrl(src);
    }
    $scope.trustHtml = function(html) {
      return $sce.trustAsHtml(html);
    }

    $scope.resetCurrentLocation = function() {

        $cordovaGeolocation.getCurrentPosition().then(

          function(data) {
   
            console.log("geolocation");
            console.log(data);

            // these are the current user's location
            $scope.coordinates.latitude = data.coords.latitude;
            $scope.coordinates.longitude = data.coords.longitude;

            // default the search to their current location
            $scope.searchData.current = true;
            $scope.searchData.mode = "current";
            $scope.searchData.location = "";
            $scope.searchData.latitude = data.coords.latitude;
            $scope.searchData.longitude = data.coords.longitude;

            // load the initial set of entries around the user
            $scope.searchNow();

          },
          function(err) {
            $scope.coordinates.loading = false;
            $scope.coordinates.error = "Unable to find your current location"
            console.log("geolocation error")
            console.log(err);
          }

        );

    }

    $scope.openSearch = function() {
      console.log("open search")

      var state = $state.current.name

      if (state != "tab.search" && state != "tab.searching") {
        $scope.searchData.lastStateName = state
      }

      $state.go("tab.search");
    }

    $scope.closeSearch = function() {
      console.log("close search")

      if ($scope.searchData.lastStateName)
        $state.go($scope.searchData.lastStateName);
      else
        $state.go("tab.list");
    }

    $scope.searchReset = function() {
          $scope.searchData.keywords = "";
          $scope.searchData.mode = "current";
          $scope.searchData.location = "";
          $scope.searchData.typeId = null;
          $scope.searchData.categoryId = null;
          $scope.searchData.tagId = null;
          $scope.searchData.distance = 50;
          $scope.searchData.max = 100;
          $scope.searchData.sortBy = "distance";
          $scope.searchData.searching = false;
          $scope.searchData.error = "";
    }

    $scope.search = function()
    {
      $scope.closeSearch();

      console.log("search");

      if ($ionicSideMenuDelegate.isOpenRight()) {
        $ionicSideMenuDelegate.toggleRight();
      }

      var sd = $scope.searchData;

      if (sd.mode == "current") {
          sd.longitude = $scope.coordinates.longitude;
          sd.latitude = $scope.coordinates.latitude;
          sd.location = "";
          $scope.searchNow();                  
      }
      else {

        // custom location?
        if (sd.location != "") {

          // have we geocoded their location?
          if (sd.longitude == null || sd.latitude == null) {
            console.log("missing/invalid coordinates")

            var geocoder = new google.maps.Geocoder();

            geocoder.geocode( { address: sd.location }, function(results, status) {
                console.log("geocoder callback")

                if (status == google.maps.GeocoderStatus.OK && status != "ZERO_RESULTS") {
                    console.log("valid coordinates", results)
                    sd.longitude = results[0].geometry.location.lng()
                    sd.latitude = results[0].geometry.location.lat()

                    $scope.searchNow();
                } else {
                    sd.error = "Unable to locate the supplied place.";
                }                    
            });

          }
          else 
            $scope.searchNow();

        }
        else {
          // No location so we're searching anywhere
          sd.longitude = null;
          sd.latitude = null;
          // distance not applicable when doing a global search
          sd.distance = 0; 

          $scope.searchNow();          
        } 
               
      }

    }

    $scope.searchNow = function() {
      console.log("searchNow")

      if ($state.current.name == "tab.searching") {
        console.log("reloading current searching state")
        $state.reload();
      }
      else {
        //$ionicViewSwitcher.nextDirection("swap");
        $ionicViewSwitcher.nextTransition("none");

        $state.go("tab.searching");
      }
      //nav-transition="none"

    }

    if ($state.current.name == "tab.home") {
      console.log("requesting current location after platform ready")

      $ionicPlatform.ready(function() {
        console.log("platform is ready");

        $cordovaGeolocation.getCurrentPosition().then(

          function(data) {
   
            console.log("geolocation");
            console.log(data);

            // these are the current user's location
            $scope.coordinates.latitude = data.coords.latitude;
            $scope.coordinates.longitude = data.coords.longitude;

            // default the search to their current location
            $scope.searchData.latitude = data.coords.latitude;
            $scope.searchData.longitude = data.coords.longitude;

            // load the initial set of entries around the user
            $scope.searchNow();

          },
          function(err) {
            $scope.coordinates.loading = false;
            $scope.coordinates.error = "Unable to find your current location"
            console.log("geolocation error")
            console.log(err);
          }

        );

      });
    }

})

.controller('SearchController', function($scope, $timeout) {

    $scope.$on('$ionicView.enter', function() {

      console.log("search view enter")
      //console.log($("div[nav-bar=active]"));
      //console.log($("div[nav-bar]"))

      $("div[nav-bar]").each(function() {
          var bar = $(this);
          var state = bar.prop("$attr-nav-bar");

          if (state == "entering" || state == "active") {

            $timeout(function() {
              console.log("timeout")

              var input = $("input[type=search]", bar);

              if (input.length > 0)
                input[0].focus();

            }, 200);
            
          }
      });

    });

})

.controller('SearchingController', function($scope, $state, $timeout, $ionicScrollDelegate, $ionicViewSwitcher, Server) {

  console.log("searching controller")

  $scope.$on('$ionicView.enter', function() {
    console.log("searching view enter")

    $scope.entries.data = null;

    var sd = $scope.searchData;

    sd.error = "";
    sd.searching = true;
    
    sd.message = "";

    if (sd.keywords.length > 0) {
      sd.message = sd.message + " matching <strong>" + sd.keywords + "</strong>";
    }

    if (sd.distance > 0) {
      sd.message = sd.message + " within <strong>" + sd.distance + "</strong> miles";
    }

    if (sd.mode == "current")
      sd.message = sd.message + " of your <strong>current location</strong>";
    else if (sd.location.length > 0) {
      if (sd.distance > 0)
        sd.message = sd.message + " of ";
      else
        sd.message = sd.message + " around ";

      sd.message = sd.message + "<strong>" + sd.location + "</strong>";
    }


    setTimeout(function() {

        Server.search(sd, function(data, total) {

          sd.searching = false;

          $scope.entries.data = data;
          $scope.entries.total = total;

          if (data && data.length > 0) {
            $ionicViewSwitcher.nextTransition("none");
            $state.go($scope.entries.lastState);    

            $timeout( function() {             
                  $ionicScrollDelegate.$getByHandle('listScroll').scrollTop();
                },
                500
             );
          }
          else {
            sd.error = "No locations found " + sd.message;
          }

        });

    }, 1000);

  });

})

.controller('HomeController', function($scope) {
})

.controller('AboutController', function($scope, $http, Globals) {
    $scope.content = {
      html: null,
      error: null
    }
    
    $http.get(Globals.contentUrl + "about?embeded=1")
      .success(function(content) {
        $scope.content.html = content;
      })
      .error(function(data, status) {
        $scope.content.error = "Error loading content: " + status;
      })
      ; 

})

.controller('AccountController', function($scope, $state, $ionicLoading, $ionicSideMenuDelegate, $ionicHistory, Users) {

    $scope.userData = {
        email: "",
        password: "",
        organization: "",
        institution: "",
        classroom: "",
        error: ""
      };

    $scope.login = function(type) {

      $ionicLoading.show({ noBackdrop: true });

      var data = $scope.userData;

      Users.login(type, data, function(data, message) {
        console.log("user login callback")
        $ionicLoading.hide()
        $scope.$parent.$parent.user = data;
        $scope.userData.error = message;

        if (data != null) {
          //$ionicSideMenuDelegate.toggleLeft();

          if (typeof analytics !== 'undefined') {
            console.log("tracking user with Google Analytics")
            analytics.setUserId(data.id)
          }

          $state.go("tab.profile");
        }
      })
    }

    $scope.logout = function() {
        Users.logout();

        $scope.$parent.$parent.user = null;
        $state.go("tab.account");
    }

})

.controller('MapCtrl', function($scope, $ionicLoading, $compile, Globals) {
    console.log("map controller")

    var map;
    var infoWindow;
    var markers = [];
    var mapOptions = {
        zoom: 5,
        center: new google.maps.LatLng(38.42077794, -82.44286000),
        mapTypeId: google.maps.MapTypeId.ROADMAP
    };

    function loadMap() {
        map = new google.maps.Map(document.getElementById("map"),  mapOptions);

        var noPoi = [
        {
            featureType: "poi",
            stylers: [
              { visibility: "off" }
            ]   
          }
        ];

        map.setOptions({styles: noPoi}); 

        // Shared panel for all of the markers
        infowindow = new google.maps.InfoWindow({
            content: "Loading...",
            maxWidth: 200
        });               
    }

    function clearMarkers() {
        if (markers) {
            for (i in markers) {
                markers[i].setMap(null);
            }
            markers.length = 0;
        }
    }

    function addMapMarker(i, lat, lon, title, info) {
        var icon = null;

        if (i >=0 && i <= 100)
            icon = "img/number_" + (i+1) + ".png";
        else
            icon = "img/number_0.png";

        var marker = new google.maps.Marker({
            position: new google.maps.LatLng(lat, lon),
            map: map,
            title: title,
            icon: icon
        });

        if (info != null) {
            //console.log("adding map marker", title, info);

            google.maps.event.addListener(marker, 'click', function() {
                console.log("map marker click")
                infowindow.setContent(info);
                infowindow.open(map,marker);
            });    
        }

        markers.push(marker);
    }

    function formatMapEntry(i, entry) {
        var thumbnail = "";

        //if (i >= 0)
        //    marker = "<img src='" + ASSET_URL + "images/marker." + (i+1) + ".png' width='27' height='36' class='marker'>";

        if (entry.thumbnail_path != "")
            thumbnail = "<img src='" + Globals.imageUrl + entry.thumbnail_path + "' align='left' width='40' height='40' alt='Thumbnail' />";

        return  "<div class='mapentry'>" + 
                    "<a href='#/tab/entry/" + entry.id + "' class='title'>" +
                      thumbnail + 
                      entry.title + 
                    "</a>" +
                "</div>";
    }

    function loadEntries() {
        console.log("MAP loadEntries")
        if (!map) return;

        var entries = $scope.entries.data;
        var latlng_pos=[];

        console.log("MAP loadEntries", entries.length)

        for (var i=0; i<entries.length; i++) {
          var entry = entries[i];

            latlng_pos[i]=new google.maps.LatLng(entry.latitude, entry.longitude);
    
            if (entry.type == null)
                entry.type = "";

            if (entry.description != null)
                entry.description = entry.description.replace(/(?:\r\n|\r|\n)/g, '<br />');

            addMapMarker(i, entry.latitude, entry.longitude, entry.title, formatMapEntry(-1, entry));
        }

        if (latlng_pos.length > 0) {
          // force the map to zoom so that all of the markers are visible
          latlngbounds = new google.maps.LatLngBounds( );
          for ( var i = 0; i < latlng_pos.length; i++ ) {
              latlngbounds.extend( latlng_pos[ i ] );
          }
          map.fitBounds( latlngbounds );  
        }
    }

    function initMap() {
        console.log("initMap")

        if ($scope.searchData.latitude != null && $scope.searchData.longitude != null) {
          console.log("adjusting map center")
          mapOptions.center = new google.maps.LatLng($scope.searchData.latitude, $scope.searchData.longitude);
        }

        console.log(mapOptions);

        loadMap(mapOptions)
        loadEntries()
    }

    initMap();

    $scope.$on('$ionicView.beforeEnter', function() {
      console.log("map view enter")
      $scope.entries.lastState = "tab.map";
    });

    $scope.$watch('entries.data', function(newValue, oldValue){
      console.log("MAP entries changed", $scope.entries.data.length)

      clearMarkers();
      loadEntries();
    });

})

.controller('ListCtrl', function($scope) {

    $scope.$on('$ionicView.beforeEnter', function() {
      console.log("map view enter")
      $scope.entries.lastState = "tab.list";
    });

})

.controller('SubmitEntryController', function($scope) {
})

.controller('EntryAddPhotoController', function($scope, $state, $stateParams, $ionicLoading, $ionicPopup, Globals, Server, Camera, Upload) {
  console.log("entry photo controller", $stateParams.entryId)

  $scope.photoData = {
      caption: "",
      uri: "", 
      error: ""
    }

  $scope.back = function()
  {
    $state.go("tab.entry", { entryId: $stateParams.entryId });
  }

  $scope.takePhoto = function() {
    console.log("take camera photo")
    Camera.takePhoto(function(uri, error) {
      console.log(uri)
      
      if (uri) 
        $scope.photoData.uri = uri;
      else 
        $scope.photoData.error = "Error: " + error;

    })
  }
  $scope.getPhoto = function() {
    console.log("get camera photo")
    Camera.getPhoto(function(uri, error) {
      console.log(uri)
      
      if (uri) 
        $scope.photoData.uri = uri;
      else 
        $scope.photoData.error = "Error: " + error;

    })
  }
  $scope.uploadPhoto = function() {
    if ($scope.user && $scope.user.id != 0) {

      $ionicLoading.show({ noBackdrop: true });

      var uploadUrl = Globals.uploadUrl + $stateParams.entryId + "&userId=" + $scope.user.id + "&caption=" + escape($scope.photoData.caption);

      console.log("upload photo", uploadUrl)
      Upload.fileTo($scope.photoData.uri, uploadUrl, function(data, error) {
        $ionicLoading.hide();

        console.log("upload photo callback")
        console.log(data)
        console.log(error)

        if (data) {
          console.log("SUCCESS!!! Going back to entry")

          //var photos = angular.fromJson(data.reponse);
          //console.log(photos);
          //$scope.entryData.entry.photos = photos;

          $scope.entryData.entry.photos.push({
              path: $scope.photoData.uri,
              caption: $scope.photoData.caption,
              created_by: $scope.user.name
          });

          $scope.back();
        } 
        else {
          console.log(error);
          $scope.photoData.error = "Error: Unable to upload and add photo.";
        }

      });
    
    }
    else {
      $scope.photoData.error = "Error: Only authenticated users can upload photos.";
    } 

  };

  console.log("entry photo controller end")
})


.controller('EntryEditController', function($scope, $state, $stateParams, Globals) {
  console.log("entry edit controller", $stateParams.entryId)
  
  $scope.back = function()
  {
    $state.go("tab.entry", { entryId: $stateParams.entryId });
  }
  $scope.externalEntryUrl = Globals.viewEntryUrl + $stateParams.entryId

  $scope.back = function()
  {
    $state.go("tab.entry", { entryId: $stateParams.entryId });
  }

})

.controller('EntryController', function($scope, $state, $stateParams, $ionicModal, $ionicPopup, $cordovaSocialSharing, Globals, Server) {
  console.log("entry controller", $stateParams.entryId)

  console.log($state);

  if (typeof analytics !== 'undefined') {
    console.log("tracking entry view in Google Analytics")
    analytics.trackEvent('Location', 'View', 'ID', $stateParams.entryId);
  }

  $scope.entryData.id = $stateParams.entryId;
  $scope.entryData.entry = null;
  $scope.entryData.error = null;
  $scope.entryData.loading = true;

  $scope.globals = Globals;

  var map;

  $scope.formatUrlForDisplay = function(url) {
    if (url != null && url.length > 0) 
      return url.replace(/\//g, " / ");
    else
      return url;
  }
  $scope.unlockMap = function(e) {
    map.setOptions({ draggable: true});
    $(e.target).hide();
  }

  Server.entry($scope.coordinates, $scope.entryData.id, function(entry, error) {
    console.log("entry get callback");
    console.log(entry);

    $scope.entryData.loading = false;

    if (entry) {
      for (var i=0; i< entry.photos.length; i++) {
        var photo = entry.photos[i];
        photo.path = Globals.imageUrl + photo.path;
      }

      $scope.entryData.entry = entry;

      setTimeout(function() {
          console.log("delayed map init")
          var mapOptions = {
              zoom: 15,
              center: new google.maps.LatLng(entry.latitude, entry.longitude),
              draggable: false,
              mapTypeId: google.maps.MapTypeId.ROADMAP
          };

          console.log("init entry map")
          console.log(document.getElementById("entryMap"));
          console.log(mapOptions);

          map = new google.maps.Map(document.getElementById("entryMap"), mapOptions);

          var noPoi = [
          {
              featureType: "poi",
              stylers: [
                { visibility: "off" }
              ]   
            }
          ];

          map.setOptions({styles: noPoi});

          var marker = new google.maps.Marker({
              position: new google.maps.LatLng(entry.latitude, entry.longitude),
              map: map,
              title: entry.title
          });

      }, 
      500);

    }
    else {
      $scope.entryData.error = "Error: Unable to load the data for this location.";
    }

  });
  
  $scope.$on('$ionicView.loaded', function(){

    console.log("entry view loaded");
    
  });

  $scope.back = function()
  {
    $state.go($scope.entries.lastState);
  }

  $scope.share = function() {
    var entry = $scope.entryData.entry;

    var message = "";
    var subject = entry.title;
    var file = Globals.imageUrl + entry.thumbnail_path;
    var link = Globals.entryUrl + entry.id;

    // Share via native share sheet
    $cordovaSocialSharing
        .share(message, subject, file, link) 
        .then(function(result) {
          // Success!
          console.log("share success!")
        }, function(err) {
          // An error occured. Show a message to the user
          $ionicPopup.alert({
            title: 'Error',
            template: 'An error occured while trying to share this location.'
          });
        });
  }

  $scope.directions = function() {

    var urlPrefix = device.platform.toLowerCase()=="ios" ? "maps:" : "http://maps.google.com/maps?";
    var url = urlPrefix + "saddr=&daddr=" + $scope.entryData.entry.latitude + "," + $scope.entryData.entry.longitude + "&sensor=TRUE";

    window.open(url, '_system', 'location=yes');

  }

  console.log("loading photo modal")

  $ionicModal.fromTemplateUrl('templates/photo-modal.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(
    function(modal) {
      console.log("loaded photo modal template")
      $scope.modal = modal;
    },
    function(err) {
      console.log("error loading photo modal template", err)
    }
  );

  $scope.openModal = function() {
    $scope.modal.show();
  };

  $scope.closeModal = function() {
    $scope.modal.hide();
  };

  //Cleanup the modal when we're done with it!
  $scope.$on('$destroy', function() {
    $scope.modal.remove();
  });

  $scope.photoSrc = null;

  $scope.showPhoto = function(src) {
    $scope.photoSrc = src;
    $scope.openModal();
  }

})

.controller('ClassroomController', function($scope, $http, Globals) {
    $scope.content = {
      html: null,
      error: null
    }
    
    $http.get(Globals.contentUrl + "intheclassroom?embeded=1")
      .success(function(content) {

        content = content.replace(RegExp('<img src=\"', "g"), '<img src="' + Globals.serverUrl)
        $scope.content.html = content;
      })
      .error(function(data, status) {
        $scope.content.error = "Error loading content: " + status;
      })
      ; 

})

.controller('FaqController', function($scope, $http, Globals) {
    $scope.content = {
      html: null,
      error: null
    }
    
    $http.get(Globals.contentUrl + "faqs?embeded=1")
      .success(function(content) {
        $scope.content.html = content;
      })
      .error(function(data, status) {
        $scope.content.error = "Error loading content: " + status;
      })
      ; 

})

.controller('SupportersController', function($scope, $http, Globals) {
    $scope.content = {
      html: null,
      error: null
    }
    
    $http.get(Globals.contentUrl + "supporters?embeded=1")
      .success(function(content) {
        $scope.content.html = content;
      })
      .error(function(data, status) {
        $scope.content.error = "Error loading content: " + status;
      })
      ; 

})

;
