angular.module('starter.services', ['ngStorage'])

.service("Globals", function Globals() {
  var globals = this;

  globals.production = true;

  if (globals.production) {
    globals.serverUrl = "http://theclio.com";
    globals.baseUrl = globals.serverUrl + "/web/";
    globals.baseDataUrl = globals.baseUrl + "app.php/data/"
  }
  else {
    globals.serverUrl = "http://localhost";
    globals.baseUrl = globals.serverUrl + "/Clio/web/";
    globals.baseDataUrl = globals.baseUrl + "app_dev.php/data/"
  }

  globals.contentUrl = globals.baseUrl + "app_dev.php/";

  globals.imageUrl = globals.baseUrl + "ul/";
  globals.uploadUrl = globals.baseDataUrl + "addPhoto?id=";
  globals.searchUrl = globals.baseDataUrl + "entriesNearPoint?";
  globals.entryUrl = globals.baseDataUrl + "entry?";
  globals.loginUrl = globals.baseDataUrl + "login?";
  globals.institutionsUrl = globals.baseDataUrl + "institutions";
  globals.classroomsUrl = globals.baseDataUrl + "classrooms";
  globals.dataUrl = globals.baseDataUrl + "data";

  globals.viewEntryUrl = globals.baseUrl + "entry?id=";

})

.factory('Server', function($http, Globals) {

    return {
      data: function(callback) {
          $http.get(Globals.dataUrl).success(function(data) {
              console.log("entries.data callback")
              callback(data);
          });  
      },
      search: function(data, callback) {

          console.log("entries.search", data.latitude, data.longitude, data.typeId, data.categoryId, data.keywords, data.location, data.distance)
          console.log(typeof data.typeId, data.typeId);
          console.log(typeof data.catgoryId, data.categoryId);
          console.log(typeof data.tagId, data.tagId);

          var params = 
            "&lat=" + data.latitude + 
            "&lon=" + data.longitude +
            "&location=" + data.location +
            "&typeId=" + (typeof data.typeId == "object" && data.typeId != null ? data.typeId.id : null) +
            "&categoryId=" + (typeof data.categoryId == "object" && data.categoryId != null ? data.categoryId.id : null) +
            "&tagId=" + (typeof data.tagId == "object" && data.tagId != null ? data.tagId.id : null) +
            "&distance=" + data.distance +
            "&max=" + data.max +
            "&sortBy=" + data.sortBy +
            "&search=" + data.keywords;

          console.log(params);

          $http.get(Globals.searchUrl + params).success(function(entries) {
              console.log("entries.search callback")

              if (entries)
                console.log(entries.length)

              var total = 0;

              if (entries && entries.length > 0) {
                total = entries[0]["total_results"];
              }
              callback(entries, total);
          });          

      },
      entry: function(coords, entryId, callback) {
          var params = "id=" + entryId + 
                      "&lat=" + coords.latitude + 
                      "&lon=" + coords.longitude;

          console.log("requesting entry data", Globals.entryUrl + params);

          $http.get(Globals.entryUrl + params)
            .success(function(data) {
              console.log("entry get success")
              callback(data);
            })
            .error(function(data, status) {
              console.log("entry get error")
              console.log(data)
              console.log(status)

              callback(null, data)
            })
            ;          

      }
    }      

})

.factory('Users', function($http, $localStorage, Globals) {

    return {
      institutions: function(callback) {
          console.log("fetch institutions")

          $http.get(Globals.institutionsUrl).success(function(data) {
              console.log("institutions callback")
              callback(data);
          });          

      }, 
      classrooms: function(callback) {
          console.log("fetch classrooms")

          $http.get(Globals.classroomsUrl).success(function(data) {
              console.log("classrooms callback")
              callback(data);
          });          

      },            
      login: function(type, data, callback) {
          console.log("user login service")
          console.log(type);
          console.log(data);

          var params = "type=" + escape(type) + 
                      "&email=" + escape(data.email) + 
                      "&organization=" + escape(data.organization) + 
                      "&classroom=" + escape(data.classroom) + 
                      "&institution=" + escape(data.institution) + 
                      "&password=" + escape(data.password);

          $http.get(Globals.loginUrl + params)
            .success(function(data) {
                console.log("users.login callback")
                console.log(data)

                if (data) {
                  // save/cache the user
                  $localStorage.user = data;                  
                  callback(data);
                }
                else
                  callback(data, "Unable to locate a user with the supplied information.")
            })
            .error(function(data, status, headers, config) {
                console.log("users.login ERROR")
                callback(null, "An error occured while trying to verify your credentials.")
            })

      },
      logout: function() {
        $localStorage.user = null;
      }

    }      

})

.factory('Camera', function($q, $cordovaCamera) {
 
    return {
 
        getPhoto: function(callback) {
          console.log("camera getPhoto")
          if (ionic.Platform.isWebView()) {
     
            var options =   {
                quality: 100, 
                destinationType: Camera.DestinationType.FILE_URI, 
                sourceType: Camera.PictureSourceType.PHOTOLIBRARY, 
                encodingType: Camera.EncodingType.JPEG
            }
     
            console.log(options);

            $cordovaCamera.getPicture(options).then(callback);
  
          }
          else {
            console.log("camera not supported")
            callback(null, 'Camera not supported in browser');
          }
 
        },
        takePhoto: function(callback) {
          console.log("camera takePhoto")

          if (ionic.Platform.isWebView()) {
     
            var options =   {
                quality: 100, 
                destinationType: Camera.DestinationType.FILE_URI, 
                sourceType: Camera.PictureSourceType.CAMERA, 
                encodingType: Camera.EncodingType.JPEG
            }

            console.log(options);
     
            $cordovaCamera.getPicture(options).then(callback);
  
          }
          else {
            console.log("camera not supported")
            callback(null, 'Camera not supported in browser');
          }
 
        }        
 
    }
 
})

.factory('Upload', function($q, $cordovaCamera, $cordovaFileTransfer) {
 
    return {
 
        fileTo: function(fileURL, serverURL, callback) {
            console.log("upload.fileTo");
            console.log(fileURL);
            console.log(serverURL);

            var uploadOptions = new FileUploadOptions();

            uploadOptions.fileKey = "file[]";
            uploadOptions.fileName = fileURL.substr(fileURL.lastIndexOf('/') + 1);
            uploadOptions.mimeType = "image/jpeg";
            uploadOptions.chunkedMode = false;
 
            $cordovaFileTransfer.upload(serverURL, fileURL, uploadOptions, true).then(

              function(data) {
                console.log("file upload success")
                console.log(data);
                callback(data);
              }, 
              function(err) {
                console.log("file upload error", err)
                callback(null, err);
              }

            );
   
        }
 
    }
 
})


;